/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.command;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.model.layout.Dimension;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.Point;
import org.fortiss.tooling.base.ui.editpart.ConnectorEditPartBase;
import org.fortiss.tooling.base.ui.editpart.LayoutedConnectionEditPartBase;
import org.fortiss.tooling.base.ui.layout.util.LayoutDataUtils;

/**
 * A {@link Command} storing a {@link ChangeBoundsRequest}.
 * 
 * @author wandinger
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public abstract class ChangeBoundsRequestCommandBase extends Command {

	/** The creation request */
	private final ChangeBoundsRequest request;

	/** Constructor */
	public ChangeBoundsRequestCommandBase(ChangeBoundsRequest request) {
		this.request = request;
	}

	/** Returns the stored request. */
	protected ChangeBoundsRequest getRequest() {
		return request;
	}

	/** {@inheritDoc} */
	@Override
	@SuppressWarnings("unchecked")
	public void execute() {
		bendpointGroupMove(request.getEditParts());
	}

	/** Moves the given node using info from {@link #getRequest()}. */
	// TODO: move this computation to LayoutDataUtil
	protected void moveNode(ILayoutedModelElement layouted) {
		Point p = LayoutDataUtils.getNodePosition(layouted);
		int dx = getRequest().getMoveDelta().x;
		int dy = getRequest().getMoveDelta().y;
		LayoutDataUtils.setNodePosition(layouted,
				LayoutDataUtils.translate(p, dx, dy));
	}

	/**
	 * Moves and resizes the given node using info from {@link #getRequest()}.
	 * The final size is returned.
	 */
	// TODO: move this computation to LayoutDataUtil
	protected Dimension moveAndResizeNode(ILayoutedModelElement layouted,
			int minWidth, int minHeight) {
		int dx = getRequest().getMoveDelta().x;
		int dy = getRequest().getMoveDelta().y;
		Point pos = LayoutDataUtils.translate(
				LayoutDataUtils.getNodePosition(layouted), dx, dy);

		int dw = getRequest().getSizeDelta().width;
		int dh = getRequest().getSizeDelta().height;
		Dimension dim = LayoutDataUtils.expand(
				LayoutDataUtils.getNodeSize(layouted), dw, dh);

		if (dim.getWidth() < minWidth) {
			if (getRequest().getMoveDelta().x > 0) {
				pos.setX(pos.getX() - minWidth - dim.getWidth());
			}
			dim.setWidth(minWidth);
		}

		if (dim.getHeight() < minHeight) {
			if (getRequest().getMoveDelta().y > 0) {
				pos.setY(pos.getY() - minHeight - dim.getHeight());
			}
			dim.setHeight(minHeight);
		}

		LayoutDataUtils.setNodePosition(layouted, pos);
		LayoutDataUtils.setNodeSize(layouted, dim);
		return dim;
	}

	/** Move bend points along with a group of selected elements */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void bendpointGroupMove(List<EditPart> parts) {
		for (EditPart part : parts) {
			// cover also sticky connectors
			if (part.getChildren().size() > 0) {
				bendpointGroupMove(part.getChildren());
			}
			// filter non connective edit parts
			if (!(part instanceof ConnectorEditPartBase)) {
				break;
			}
			ConnectorEditPartBase connector = (ConnectorEditPartBase) part;
			for (LayoutedConnectionEditPartBase connection : (List<LayoutedConnectionEditPartBase>) connector
					.getSourceConnections()) {
				if (connection.getTarget().getSelected() != EditPart.SELECTED_NONE) {
					LayoutDataUtils.changeAllConnectionPoints(
							(ILayoutedModelElement) connection.getModel(),
							request.getMoveDelta().x, request.getMoveDelta().y);
				}
			}
		}
	}
}
